@@ -8,36 +8,35 @@ BaseComplexPrompt ← FuzzyPrompt
         ↑
 BaseListPrompt ← ListPrompt, ExpandPrompt ...
 """
-
-from abc import ABC, abstractmethod
+import os
 import re
+from abc import ABC, abstractmethod
 from typing import Any, Callable, Dict, List, NamedTuple, Tuple, Union
 
 from prompt_toolkit.application import Application
 from prompt_toolkit.enums import EditingMode
 from prompt_toolkit.filters import IsDone
 from prompt_toolkit.filters.base import Condition, FilterOrBool
-from prompt_toolkit.key_binding.key_bindings import KeyBindings
+from prompt_toolkit.key_binding.key_bindings import KeyBindings, KeyHandlerCallable
 from prompt_toolkit.keys import Keys
-from prompt_toolkit.layout.containers import (
-    ConditionalContainer,
-    Float,
-    FloatContainer,
-    HSplit,
-    Window,
-)
+from prompt_toolkit.layout.containers import ConditionalContainer, HSplit, Window
 from prompt_toolkit.layout.controls import FormattedTextControl
 from prompt_toolkit.layout.dimension import Dimension, LayoutDimension
 from prompt_toolkit.layout.layout import Layout
 from prompt_toolkit.styles.style import Style
 from prompt_toolkit.validation import ValidationError, Validator
 
-from InquirerPy.enum import ACCEPTED_KEYBINDINGS, INQUIRERPY_KEYBOARD_INTERRUPT
+from InquirerPy.enum import INQUIRERPY_KEYBOARD_INTERRUPT
 from InquirerPy.exceptions import InvalidArgument, RequiredKeyNotFound
 from InquirerPy.separator import Separator
-from InquirerPy.utils import calculate_height, get_style
+from InquirerPy.utils import InquirerPyStyle, SessionResult, calculate_height, get_style
 
-__all__ = ["BaseSimplePrompt", "BaseComplexPrompt", "BaseListPrompt"]
+__all__ = [
+    "BaseSimplePrompt",
+    "BaseComplexPrompt",
+    "BaseListPrompt",
+    "InquirerPyUIControl",
+]
 
 
 class BaseSimplePrompt(ABC):
@@ -50,49 +49,57 @@ class BaseSimplePrompt(ABC):
     a call of `self.session = PromptSession(...)`.
 
     :param message: the question message to display
-    :type message: str
+    :type message: Union[str, Callable[[SessionResult], str]]
     :param style: the style dictionary to apply
-    :type style: Dict[str, str]
-    :param editing_mode: default keybinding mode
-    :type editing_mode: str
+    :type style: InquirerPyStyle
+    :param vi_mode: use vi kb for the prompt
+    :type vi_mode: str
     :param qmark: the custom qmark to display infront of the question
     :type qmark: str
     :param validate: a callable or Validator instance to validate user input
-    :type validate: Union[Callable[[str], bool], Validator]
+    :type validate: Union[Callable[[Any], bool], Validator]
     :param invalid_message: message to display when input is invalid
     :type invalid_message: str
     :param transformer: a callable to transform the result, this is visual effect only
-    :type transformer: Callable[[str], Any]
+    :type transformer: Callable[[Any], Any]
     :param filter: a callable to filter the result, updating the user input before returning the result
     :type filter: Callable[[Any], Any]
+    :param session_result: the current session result, this is used by callable message and choices
+        to generate dynamic values. If using alternate syntax, skip this value.
+    :type session_result: SessionResult
     """
 
     def __init__(
         self,
-        message: str,
-        style: Dict[str, str] = None,
-        editing_mode: str = "default",
+        message: Union[str, Callable[[SessionResult], str]],
+        style: InquirerPyStyle = None,
+        vi_mode: bool = False,
         qmark: str = "?",
-        validate: Union[Callable[[str], bool], Validator] = None,
+        validate: Union[Callable[[Any], bool], Validator] = None,
         invalid_message: str = "Invalid input",
-        transformer: Callable[[str], Any] = None,
+        transformer: Callable[[Any], Any] = None,
         filter: Callable[[Any], Any] = None,
+        session_result: SessionResult = None,
+        default: Any = "",
     ) -> None:
         """Construct the base class for simple prompts."""
-        self._message = message
-        self._style = Style.from_dict(style or get_style())
+        self._result = session_result or {}
+        self._message = message if not isinstance(message, Callable) else message(self._result)  # type: ignore
+        self._default = (
+            default if not isinstance(default, Callable) else default(self._result)
+        )
+        self._style = Style.from_dict(style.dict if style else get_style().dict)
         self._qmark = qmark
         self._status = {"answered": False, "result": None}
         self._kb = KeyBindings()
         self._lexer = "class:input"
         self._transformer = transformer
         self._filter = filter
-        try:
-            self._editing_mode = ACCEPTED_KEYBINDINGS[editing_mode]
-        except KeyError:
-            raise InvalidArgument(
-                "editing_mode must be one of 'default' 'emacs' 'vim'."
-            )
+        self._editing_mode = (
+            EditingMode.VI
+            if vi_mode or bool(os.getenv("INQUIRERPY_VI_MODE", False))
+            else EditingMode.EMACS
+        )
         if isinstance(validate, Validator):
             self._validator = validate
         else:
@@ -118,6 +125,47 @@ class BaseSimplePrompt(ABC):
         """Set status value of the prompt."""
         self._status = value
 
+    def register_kb(
+        self, *keys: Union[Keys, str], filter: FilterOrBool = True
+    ) -> Callable[[KeyHandlerCallable], KeyHandlerCallable]:
+        """Decorate keybinding registration function.
+
+        Format all alt related keybindings.
+
+        Due to `prompt_toolkit` doesn't process alt related keybindings,
+        it requires alt-ANY to "escape" + "ANY".
+
+        Check a list of keys argument if they are alt related, change
+        them to escape.
+
+        :param keys: keys to bind into the keybindings
+        :type keys: Union[Keys, str]
+        :param filter: condition of whether this keybinding should be active
+        :type filter: FilterOrBool
+        :return: a decorator that should be applied to the function thats intended
+            to be active when the keys being pressed
+        :rtype: Callable[[KeyHandlerCallable], KeyHandlerCallable]
+        """
+        alt_pattern = re.compile(r"^alt-(.*)")
+
+        def decorator(func: KeyHandlerCallable) -> KeyHandlerCallable:
+            formatted_keys = []
+            for key in keys:
+                match = alt_pattern.match(key)
+                if match:
+                    formatted_keys.append("escape")
+                    formatted_keys.append(match.group(1))
+                else:
+                    formatted_keys.append(key)
+
+            @self._kb.add(*formatted_keys, filter=filter)
+            def executable(event) -> None:
+                func(event)
+
+            return executable
+
+        return decorator
+
     @abstractmethod
     def _get_prompt_message(
         self, pre_answer: Tuple[str, str], post_answer: Tuple[str, str]
@@ -148,7 +196,10 @@ class BaseSimplePrompt(ABC):
                 display_message.append(
                     post_answer
                     if not self._transformer
-                    else ("class:answer", " %s" % self._transformer(post_answer[1][1:]))
+                    else (
+                        "class:answer",
+                        " %s" % self._transformer(self.status["result"]),
+                    )
                 )
             else:
                 display_message.append(pre_answer)
@@ -170,22 +221,28 @@ class InquirerPyUIControl(FormattedTextControl):
     Dynamically adapt to user input and update formatted text.
 
     :param choices: list of choices to display as the content
-    :type choices: Union[Callable[[], List[Any]], List[Any]],
+    :type choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
     :param default: default value, will impact the cursor position
     :type default: Any
     """
 
     def __init__(
         self,
-        choices: Union[Callable[[], List[Any]], List[Any]],
+        choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
         default: Any = None,
+        session_result: SessionResult = None,
     ) -> None:
         """Initialise choices and construct a FormattedTextControl object."""
+        self._session_result = session_result or {}
         self._selected_choice_index: int = 0
         self._choice_func = None
-        self._default = default
         self._loading = False
         self._raw_choices = []
+        self._default = (
+            default
+            if not isinstance(default, Callable)
+            else default(self._session_result)
+        )
         if isinstance(choices, Callable):
             self._loading = True
             self._choices = []
@@ -202,8 +259,12 @@ class InquirerPyUIControl(FormattedTextControl):
         """Retrieve the callable choices and format them.
 
         Should be called in the `after_render` call in `Application`.
+
+        :param session_result: the current result of the prompt session,
+            if using alternate syntax, skip this value
+        :type session_result: SessionResult
         """
-        self._raw_choices = self._choice_func()  # type: ignore
+        self._raw_choices = self._choice_func(self._session_result)  # type: ignore
         self.choices = self._get_choices(self._raw_choices, self._default)
         self._loading = False
         self._safety_check()
@@ -372,19 +433,18 @@ class BaseComplexPrompt(BaseSimplePrompt):
 
     def __init__(
         self,
-        message: str,
-        style: Dict[str, str] = None,
-        editing_mode: str = "default",
+        message: Union[str, Callable[[SessionResult], str]],
+        style: InquirerPyStyle = None,
+        vi_mode: bool = False,
         qmark: str = "?",
         instruction: str = "",
-        transformer: Callable[[str], Any] = None,
+        transformer: Callable[[Any], Any] = None,
         filter: Callable[[Any], Any] = None,
-        height: Union[int, str] = None,
-        max_height: Union[int, str] = None,
-        validate: Union[Callable[[str], bool], Validator] = None,
+        validate: Union[Callable[[Any], bool], Validator] = None,
         invalid_message: str = "Invalid input",
         multiselect: bool = False,
         keybindings: Dict[str, List[Dict[str, Union[str, FilterOrBool]]]] = None,
+        session_result: SessionResult = None,
     ) -> None:
         """Initialise the Application with Layout and keybindings."""
         if not keybindings:
@@ -392,12 +452,13 @@ class BaseComplexPrompt(BaseSimplePrompt):
         super().__init__(
             message=message,
             style=style,
-            editing_mode=editing_mode,
+            vi_mode=vi_mode,
             qmark=qmark,
             transformer=transformer,
             filter=filter,
             invalid_message=invalid_message,
             validate=validate,
+            session_result=session_result,
         )
         self._content_control: InquirerPyUIControl
         self._instruction = instruction
@@ -405,9 +466,6 @@ class BaseComplexPrompt(BaseSimplePrompt):
         self._multiselect = multiselect
         self._rendered = False
         self._invalid = False
-        self._dimmension_height, self._dimmension_max_height = calculate_height(
-            height, max_height
-        )
         self._application: Application
 
         @Condition
@@ -471,9 +529,14 @@ class BaseComplexPrompt(BaseSimplePrompt):
             "toggle-all-true": [{"func": self._toggle_all, "args": [True]}],
             "toggle-all-false": [{"func": self._toggle_all, "args": [False]}],
         }
+        self._non_multiselect_action = {"down", "up"}
 
-        @self._format_keys
         def keybinding_factory(keys, filter, action):
+            if not isinstance(keys, list):
+                keys = [keys]
+            if action not in self._non_multiselect_action:
+                filter = filter & self._multiselect
+
             @self._register_kb(*keys, filter=filter)
             def _(event):
                 for method in self._kb_func_lookup[action]:
@@ -489,19 +552,19 @@ class BaseComplexPrompt(BaseSimplePrompt):
 
     def _register_kb(
         self, *keys: Union[Keys, str], filter: FilterOrBool = True
-    ) -> Callable:
+    ) -> Callable[[KeyHandlerCallable], KeyHandlerCallable]:
         """Decorate keybinding registration function.
 
         Ensure that invalid state is cleared on next
         keybinding entered.
         """
 
-        def decorator(func: Callable) -> Callable:
-            @self._kb.add(*keys, filter=filter)
+        def decorator(func: KeyHandlerCallable) -> KeyHandlerCallable:
+            @self.register_kb(*keys, filter=filter)
             def executable(event):
                 if self._invalid:
                     self._invalid = False
-                return func(event)
+                func(event)
 
             return executable
 
@@ -528,9 +591,22 @@ class BaseComplexPrompt(BaseSimplePrompt):
         post_answer = ("class:answer", " %s" % self.status["result"])
         return super()._get_prompt_message(pre_answer, post_answer)
 
-    def execute(self) -> Any:
-        """Execute the application and get the result."""
+    def execute(self, raise_keyboard_interrupt: bool = True) -> Any:
+        """Execute the application and get the result.
+
+        :param raise_keyboard_interrupt: raise kbi exception when user hit 'c-c'
+        :type raise_keyboard_interrupt: bool
+        :return: user selected value
+        :rtype: Any
+        """
         result = self.application.run()
+        if result == INQUIRERPY_KEYBOARD_INTERRUPT:
+            if raise_keyboard_interrupt and not os.getenv(
+                "INQUIRERPY_NO_RAISE_KBI", False
+            ):
+                raise KeyboardInterrupt
+            else:
+                result = None
         if not self._filter:
             return result
         return self._filter(result)
@@ -579,7 +655,10 @@ class BaseComplexPrompt(BaseSimplePrompt):
         if self._multiselect:
             return [choice["value"] for choice in self.selected_choices]
         else:
-            return self.content_control.selection["value"]
+            try:
+                return self.content_control.selection["value"]
+            except IndexError:
+                return ""
 
     @property
     def selected_choices(self) -> List[Any]:
@@ -610,37 +689,6 @@ class BaseComplexPrompt(BaseSimplePrompt):
         """Setter for `self._application`."""
         self._application = value
 
-    def _format_keys(self, func) -> Callable:
-        """Format all alt related keybindings.
-
-        Due to `prompt_toolkit` doesn't process alt related keybindings,
-        it requires alt-ANY to "escape" + "ANY".
-
-        Check a list of keys argument if they are alt related, change
-        them to escape.
-
-        Apply multiselect filter if action is related to multiselect action.
-        """
-        alt_pattern = re.compile(r"^alt-(.*)")
-        non_multiselect_action = {"down", "up"}
-
-        def wrapper(keys, filter, action):
-            formatted_keys = []
-            if not isinstance(keys, list):
-                keys = [keys]
-            for key in keys:
-                match = alt_pattern.match(key)
-                if match:
-                    formatted_keys.append("escape")
-                    formatted_keys.append(match.group(1))
-                else:
-                    formatted_keys.append(key)
-            if action not in non_multiselect_action:
-                filter = filter & self._multiselect
-            func(formatted_keys, filter, action)
-
-        return wrapper
-
     @abstractmethod
     def _handle_enter(self, event) -> None:
         """Handle event when user input enter key."""
@@ -676,17 +724,17 @@ class BaseListPrompt(BaseComplexPrompt):
     Upon entering the answer, update the first window's formatted text.
 
     :param message: question to display to the user
-    :type message: str
+    :type message: Union[str, Callable[[SessionResult], str]]
     :param style: style to apply to the prompt
-    :type style: Dict[str, str]
-    :param editing_mode: controls the key_binding
-    :type editing_mode: str
+    :type style: InquirerPyStyle
+    :param vi_mode: use vi kb for the prompt
+    :type vi_mode: bool
     :param qmark: question mark to display
     :type qmark: str
     :param instruction: instruction to display after the question message
     :type instruction: str
     :param transformer: a callable to transform the result, this is visual effect only
-    :type transformer: Callable[[str], Any]
+    :type transformer: Callable[[Any], Any]
     :param filter: a callable to filter the result, updating the user input before returning the result
     :type filter: Callable[[Any], Any]
     :param height: preferred height of the choice window
@@ -694,36 +742,40 @@ class BaseListPrompt(BaseComplexPrompt):
     :param max_height: max height choice window should reach
     :type max_height: Union[str, int]
     :param validate: a callable or Validator instance to validate user selection
-    :type validate: Union[Callable[[str], bool], Validator]
+    :type validate: Union[Callable[[Any], bool], Validator]
     :param invalid_message: message to display when input is invalid
     :type invalid_message: str
     :param multiselect: enable multiselect mode
     :type multiselect: bool
     :param keybindings: custom keybindings to apply
     :type keybindings: Dict[str, List[Dict[str, Union[str, FilterOrBool]]]]
+    :param show_cursor: display cursor at the end of the prompt
+    :type show_cursor: bool
     """
 
     def __init__(
         self,
-        message: str,
-        style: Dict[str, str] = None,
-        editing_mode: str = "default",
+        message: Union[str, Callable[[SessionResult], str]],
+        style: InquirerPyStyle = None,
+        vi_mode: bool = False,
         qmark: str = "?",
         instruction: str = "",
-        transformer: Callable[[str], Any] = None,
+        transformer: Callable[[Any], Any] = None,
         filter: Callable[[Any], Any] = None,
         height: Union[int, str] = None,
         max_height: Union[int, str] = None,
-        validate: Union[Callable[[str], bool], Validator] = None,
+        validate: Union[Callable[[Any], bool], Validator] = None,
         invalid_message: str = "Invalid input",
         multiselect: bool = False,
         keybindings: Dict[str, List[Dict[str, Union[str, FilterOrBool]]]] = None,
+        show_cursor: bool = True,
+        session_result: SessionResult = None,
     ) -> None:
         """Initialise the Application with Layout and keybindings."""
         super().__init__(
             message=message,
             style=style,
-            editing_mode=editing_mode,
+            vi_mode=vi_mode,
             qmark=qmark,
             transformer=transformer,
             filter=filter,
@@ -731,9 +783,11 @@ class BaseListPrompt(BaseComplexPrompt):
             validate=validate,
             multiselect=multiselect,
             instruction=instruction,
-            height=height,
-            max_height=max_height,
             keybindings=keybindings,
+            session_result=session_result,
+        )
+        self._dimmension_height, self._dimmension_max_height = calculate_height(
+            height, max_height
         )
 
         self.layout = HSplit(
@@ -741,41 +795,41 @@ class BaseListPrompt(BaseComplexPrompt):
                 Window(
                     height=LayoutDimension.exact(1),
                     content=FormattedTextControl(
-                        self._get_prompt_message, show_cursor=False
+                        self._get_prompt_message_with_cursor
+                        if show_cursor
+                        else self._get_prompt_message,
+                        show_cursor=show_cursor,
                     ),
                 ),
                 ConditionalContainer(
-                    FloatContainer(
-                        content=Window(
-                            content=self.content_control,
-                            height=Dimension(
-                                max=self._dimmension_max_height,
-                                preferred=self._dimmension_height,
-                            ),
+                    Window(
+                        content=self.content_control,
+                        height=Dimension(
+                            max=self._dimmension_max_height,
+                            preferred=self._dimmension_height,
                         ),
-                        floats=[
-                            Float(
-                                ConditionalContainer(
-                                    Window(
-                                        FormattedTextControl(
-                                            [
-                                                (
-                                                    "class:validation-toolbar",
-                                                    self._invalid_message,
-                                                )
-                                            ]
-                                        ),
-                                        dont_extend_height=True,
-                                    ),
-                                    filter=self._is_invalid,
-                                ),
-                                bottom=0,
-                                left=0,
-                            )
-                        ],
+                        dont_extend_height=True,
                     ),
                     filter=~IsDone() & ~self._is_loading,
                 ),
+                ConditionalContainer(
+                    Window(FormattedTextControl([("", "")])),
+                    filter=~IsDone(),  # force validation bar to stay bottom
+                ),
+                ConditionalContainer(
+                    Window(
+                        FormattedTextControl(
+                            [
+                                (
+                                    "class:validation-toolbar",
+                                    self._invalid_message,
+                                )
+                            ]
+                        ),
+                        dont_extend_height=True,
+                    ),
+                    filter=self._is_invalid & ~IsDone(),
+                ),
             ]
         )
 
@@ -786,6 +840,20 @@ class BaseListPrompt(BaseComplexPrompt):
             after_render=self._after_render,
         )
 
+    def _get_prompt_message_with_cursor(self) -> List[Tuple[str, str]]:
+        """Obtain the prompt message to display.
+
+        Introduced a new method instead of using the `_get_prompt_message`
+        due to `expand` and `rawlist` make changes after calling `super()._get_prompt_message()`.
+
+        This ensures that cursor is always at the end of the window no matter
+        when the changes is made to the `_get_prompt_message`.
+        """
+        message = self._get_prompt_message()
+        message.append(("[SetCursorPosition]", ""))
+        message.append(("", " "))  # [SetCursorPosition] require char behind it
+        return message
+
     def _toggle_choice(self) -> None:
         """Toggle the `enabled` status of the choice."""
         self.content_control.selection["enabled"] = not self.content_control.selection[
